import React, { Component } from 'react'
import { connect } from 'react-redux'
import Base from '../common/Base'
import ContainersTable from '../tables/ContainersTable'
import { get, post, notify } from '../../actions'
import { DOCKER } from '../../constants/ResourceNames'
import { forEach, concat } from 'lodash'

class ContainerList extends Component {

    /**
     * 
     */
    componentDidMount() {
        this.props.loadData()
    }

    /**
     * 
     */
    onAction(e, action, ids) {
        this.props.sendData(action, ids)
    }

    /**
     * 
     */
    render() {
        const columns = [
            {
                title: 'Identificador',
                key: 'id'
            },
            {
                title: 'Nombre',
                key: 'name'
            },
            {
                title: 'Estado',
                key: 'status',
                options: {
                    exited: {
                        text: 'Offline'
                    },
                    running: {
                        text: 'Online'
                    }
                }
            }
        ]

        return (
            <Base title={this.props.title}>
                <ContainersTable 
                    columns={columns} 
                    rows={this.props.data} 
                    onAction={(e, action, ids) => this.onAction(e, action, ids)}
                    notify={(message) => this.props.notify(message)} />
            </Base>
        )
    }
}

/**
 * 
 * @param {*} state 
 * @param {*} props 
 */
const mapStateToProps = (state, props) => {
    return {
        data: state.containers
    }
}

/**
 * 
 * @param {*} state 
 * @param {*} props 
 */
const mapDispatchToProps = (dispatch, props) => ({
    /**
     * 
     */
    loadData() {
        dispatch(get(`${DOCKER}container/all/`))
    },
    /**
     * 
     * @param {*} action 
     * @param {*} ids 
     */
    sendData(action, ids) {
        let urls = []

        forEach(ids, id => {
            urls = concat(urls, `${DOCKER}container/${action}/${id}/`)
        })

        dispatch(post(urls))
    },
    /**
     * 
     * @param {*} text 
     */
    notify(message) {
        dispatch(notify(message))
    }
})

export default connect(mapStateToProps, mapDispatchToProps)(ContainerList)