import React, { Component } from 'react'
import Button from 'material-ui/Button'
import Base from '../common/Base'
import Paper from 'material-ui/Paper'
import Table, { TableHead, TableBody, TableRow, TableCell } from 'material-ui/Table'
import Dialog, { DialogActions, DialogContent, DialogContentText, DialogTitle } from 'material-ui/Dialog'
import TextField from 'material-ui/TextField'
import { isEqual, isEmpty } from 'lodash'
import { ODOO } from '../../constants/ResourceNames'
import { post } from '../../actions'
import { connect } from 'react-redux'
import { withStyles } from 'material-ui/styles'

const styles = theme => ({
    gap: {
        marginTop: 25
    }
})

class TasksList extends Component {
    constructor(props) {
        super(props)

        this.state = {
            isDialogOpened: false,
            isDialogValid: true,
            name: '',
            nameConfirmation: ''
        }
    }

    /**
     * 
     */
    handleOpenDialog = e => {
        this.setState({
            isDialogOpened: true
        })
    }

    /**
     * 
     */
    handleCloseDialog = e => {
        this.setState({
            isDialogOpened: false,
            isDialogValid: true,
            name: '',
            nameConfirmation: ''
        })
    }
    
    /**
    * 
    */
    handleChangeName = e => {
        if (isEqual(e.target.id, 'name')) {
            this.setState({
                name: e.target.value
            }, this.handleStateChange)
        }

        if (isEqual(e.target.id, 'nameConfirmation')) {
            this.setState({
                nameConfirmation: e.target.value
            }, this.handleStateChange)
        }
    }

    /**
     * 
     */
    handleStateChange = () => {
        if (isEqual(this.state.name, this.state.nameConfirmation)) {
            this.setState({
                isDialogValid: true
            })
        } else {
            this.setState({
                isDialogValid: false
            })
        }
    }

    /**
     * 
     */
    handleAcceptDialog = e => {
        const { name, nameConfirmation } = this.state
        
        if (isEmpty(name) || isEmpty(nameConfirmation)) {
            return
        }

        if (isEqual(name, nameConfirmation)) {
            this.props.createOdoo(name)

            this.setState({
                isDialogOpened: false,
                name: '',
                nameConfirmation: ''
            })

        }
    }

    /**
     * 
     */
    render() {
        const { isDialogOpened, isDialogValid } = this.state

        return (
            <Base title={this.props.title}>
                <Paper>
                    <Table>
                        <TableHead>
                            <TableRow>
                                <TableCell>Nombre</TableCell>
                                <TableCell>Última ejecución</TableCell>
                                <TableCell>Usuario</TableCell>
                                <TableCell />
                            </TableRow>
                        </TableHead>
                        <TableBody>
                            <TableRow>
                                <TableCell>Crear contenedor Odoo</TableCell>
                                <TableCell>Nunca</TableCell>
                                <TableCell>Anónimo</TableCell>
                                <TableCell>
                                    <Button variant='raised' color='primary' data-action='create' onClick={this.handleOpenDialog}>Ejecutar</Button>
                                </TableCell>
                            </TableRow>
                        </TableBody>
                    </Table>
                </Paper>
                <Dialog open={isDialogOpened} onClose={this.handleCloseDialog}>
                    <DialogTitle>Confirmar</DialogTitle>
                    <DialogContent>
                        <DialogContentText>Estás solicitando crear un nuevo contenedor Odoo. Tenga en cuenta que ésta tarea puede tardar unos instantes dependiendo del tráfico de red y del uso de los recursos del servidor</DialogContentText>
                        <TextField 
                            id='name'
                            margin='dense' 
                            label='Nombre del sistema'
                            type='text' 
                            autoComplete='off' 
                            onKeyUp={this.handleChangeName}
                            autoFocus 
                            fullWidth
                         />
                        <TextField
                            id='nameConfirmation'
                            margin='dense' 
                            label='Repita el nombre del sistema'
                            type='text' 
                            autoComplete='off' 
                            error={!isDialogValid}
                            onKeyUp={this.handleChangeName}
                            fullWidth
                         />
                    </DialogContent>
                    <DialogActions>
                        <Button color='primary' onClick={this.handleCloseDialog}>Cancelar</Button>
                        <Button color='primary' onClick={this.handleAcceptDialog}>Aceptar</Button>
                    </DialogActions>
                </Dialog>
            </Base>
        )
    }
}

/**
 * 
 * @param {*} dispatch 
 * @param {*} props 
 */
const mapDispatchToProps = (dispatch, props) => ({
    /**
     * 
     */
    createOdoo(name) {
        dispatch(post(`${ODOO}create/`, {
            name
        }))
    }
})

TasksList = withStyles(styles)(TasksList)

export default connect(null, mapDispatchToProps)(TasksList)