import React, { Component } from 'react'
import { withStyles } from 'material-ui/styles'
import Paper from 'material-ui/Paper'
import Button from 'material-ui/Button'
import Table, { TableHead, TableBody, TableRow, TableCell} from 'material-ui/Table'
import Checkbox from 'material-ui/Checkbox'
import { indexOf, concat, difference, slice, isEqual, size, map, pick, get, has } from 'lodash';

const styles = theme => ({
    root: {
        width: '100%'
    },
    buttons: {
        marginRight: theme.spacing.unit / 2,
        marginBottom: theme.spacing.unit * 3
    },
    tableRow: {
        '&:hover': {
            backgroundColor: theme.palette.grey[200],
            cursor: 'pointer'
        }
    }
})

class ContainersTable extends Component {

    constructor(props) {
        super(props)

        this.state = {
            selectedRows: []
        }
    }

    /**
     * 
     * @param {*} e 
     */
    onAction(e, action) {
        if (isEqual(size(this.state.selectedRows), 0) && has(this.props, 'notify')) {
            this.props.notify('No se ha seleccionado ningún elemento')
            return;
        }

        this.props.onAction(e, action, this.state.selectedRows);
        this.clearSelection()
    }

    /**
     * 
     */
    onSelectAll = (e) => {
        this.setState({
            selectedRows: isEqual(size(this.state.selectedRows), size(this.props.rows)) ? [] : map(this.props.rows, item => {
                return get(pick(item, 'id'), 'id')
            })
        })        
    }

    /**
     * 
     */
    onSelect(e, id) {
        let index = indexOf(this.state.selectedRows, id)

        this.setState({
            selectedRows: index === -1 ? concat(this.state.selectedRows, id) : difference(this.state.selectedRows, slice(this.state.selectedRows, index, index + 1))
        })
    }

    /**
     * 
     */
    clearSelection() {
        this.setState({
            selectedRows: []
        })
    }

    /**
     * 
     */
    render() {
        const { classes, columns, rows } = this.props
        const { selectedRows } = this.state

        return (
            <div>
                <Button variant='raised' color='primary' size='small' className={classes.buttons} onClick={(e) => this.onAction(e, 'start')}>Arrancar</Button>
                <Button variant='raised' color='primary' size='small' className={classes.buttons} onClick={(e) => this.onAction(e, 'stop')}>Parar</Button>
                <Button variant='raised' color='primary' size='small' className={classes.buttons} onClick={(e) => this.onAction(e, 'restart')}>Reiniciar</Button>
                <Paper className={classes.root}>
                    <Table>
                        <TableHead>
                            <TableRow>
                                <TableCell children={<Checkbox color='primary' onClick={(e) => this.onSelectAll(e)} checked={isEqual(size(selectedRows), size(this.props.rows))} />} />
                                {columns.map(column => 
                                    <TableCell key={column.key} children={column.title} />
                                )}
                            </TableRow>
                        </TableHead>
                        <TableBody>
                            {rows.map(item => 
                                <TableRow key={item.id} className={classes.tableRow}>
                                    <TableCell children={<Checkbox color='primary' onClick={(e) => this.onSelect(e, item.id)} checked={!isEqual(indexOf(selectedRows, item.id), -1)} />} />
                                    {columns.map(column => 
                                        <TableCell key={column.key} children={!column.options ? item[column.key] : column.options[item[column.key]].text} />
                                    )}
                                </TableRow>
                            )}
                        </TableBody>
                    </Table>
                </Paper>
            </div>
        )
    }
}

export default withStyles(styles, {withTheme: true})(ContainersTable)
